/*
 * RationalNumber.h
 *
 *  Interface of ADT Rational Number
 */

#ifndef RATIONALNUMBER_H_
#define RATIONALNUMBER_H_

#include <iostream>
#include <string>
#include <stdexcept>

using namespace std;

namespace Rationals {

// ADT invariant: denomunator > 0 && gcd(numerator, denominator) = 1
class Rational {
public:
	Rational() : num(0), denom(1) {}
	Rational(int n) : num, denom(1) {}
	Rational( int n, int d);

	// Another way to generate three constructors
	//Rational(int num = 0, int denom = 1);

	// Assignment from Integer Type
	Rational& operator=(int n) { return assign(n, 1); }

	// Assign in place
	Rational& assign( int n, int d);

	// Accessors
	int getNumerator() const;
	int getDenominator() const;

	// Arithmetic operations
	Rational operator +( const Rational& ) const;
	Rational operator -( const Rational& ) const;
	Rational operator *( const Rational& ) const;
	Rational operator /( const Rational& ) const;

	// Unary plus and minus
	Rational operator +() const { return *this; }
	Rational operator -() const { Rational temp(-this->num, this->denom); return temp; }

	// Increment and decrement
	Rational operator ++() { num += denom; return *this; }
	Rational operator --() { num -= denom; return *this; }

	Rational operator ++( int n );
	Rational operator --( int n );

	// Arithmetic assignment operators
	Rational& operator += (const Rational& other);
	Rational& operator -= (const Rational& other);
	Rational& operator *= (const Rational& other);
	Rational& operator /= (const Rational& other);

	// Arithmetic assignment operators - second operand is int type
	Rational& operator += (int other) { return operator += (Rational(other)); }
	Rational& operator -= (int other) { return operator -= (Rational(other)); }
	Rational& operator *= (int other) { return operator *= (Rational(other)); }
	Rational& operator /= (int other) { return operator /= (Rational(other)); }

	// Comparison operators
	bool operator < (const Rational& other) const;
	bool operator > (const Rational& other) const;
	bool operator == (const Rational& other) const { return num == other.num && denom == other.denom; }

	// Comparison operators - second operand is int type
	bool operator < (int other) const { return *this < Rational(other); }
	bool operator > (int other) const { return *this > Rational(other); }
	bool operator == (int other) const { return num == other && denom == 1; }

	// Printing Rationals
	void print( ostream& ) const;
	void print() { print(cout); }
	Rational get( istream& );

private:
	int num;
	int denom;
	void normalize();
};

// output operator
inline ostream& operator << (ostream& os, const Rational& r) {
	r.print(os);
	return os;
}

inline istream& operator >>(istream& is, Rational& r) {
	r = r.get(is);
	return is;
}

#endif /* RATIONALNUMBER_H_ */
}